# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011-2021 Hiroshi Miura <miurahr@linux.com>
#
import enum
from typing import Dict, List, Tuple

import jaconv

from .kanji import JConv
from .properties import Ch
from .scripts import A2, H2, IConv, K2, Sym2


class PyKakasiException(Exception):
    pass


class UnknownCharacterException(PyKakasiException):
    pass


class _TYPE(enum.Enum):
    KANJI = 1
    KANA = 2
    HIRAGANA = 3
    SYMBOL = 4
    ALPHA = 5


class Kakasi:
    """Kakasi is a conversion class for Japanese text."""

    def __init__(self):
        self._jconv = JConv()
        self._iconv = IConv()

    @classmethod
    def normalize(cls, text):
        return jaconv.normalize(text)

    def convert(self, text: str) -> List[Dict[str, str]]:
        """Convert input text to dictionary contains KANA, HIRA and romaji results."""

        if len(text) == 0:
            return [
                {
                    "orig": "",
                    "kana": "",
                    "hira": "",
                    "hepburn": "",
                    "passport": "",
                    "kunrei": "",
                }
            ]

        original_text = ""
        kana_text = ""
        _result = []
        i = 0
        prev_type = _TYPE.KANJI
        output_flag: Tuple[bool, bool, bool] = (False, False, False)

        while i < len(text):
            # output_flag
            # means (output buffer?, output text[i]?, copy and increment i?)
            # possible (False, True, True), (True, False, False), (True, True, True)
            #          (False, False, True)
            if text[i] in Ch.endmark:
                prev_type = _TYPE.SYMBOL
                output_flag = (True, True, True)
            elif text[i] in Ch.long_symbols:
                # FIXME: special case
                output_flag = (False, False, True)
            elif Sym2.isRegion(text[i]):
                if prev_type != _TYPE.SYMBOL:
                    output_flag = (True, False, True)
                else:
                    output_flag = (False, True, True)
                prev_type = _TYPE.SYMBOL
            elif K2.isRegion(text[i]):
                output_flag = (prev_type != _TYPE.KANA, False, True)
                prev_type = _TYPE.KANA
            elif H2.isRegion(text[i]):
                output_flag = (prev_type != _TYPE.HIRAGANA, False, True)
                prev_type = _TYPE.HIRAGANA
            elif A2.isRegion(text[i]):
                output_flag = (prev_type != _TYPE.ALPHA, False, True)
                prev_type = _TYPE.ALPHA
            elif self._jconv.isRegion(text[i]):
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                t, ln = self._jconv.convert(text[i:])
                prev_type = _TYPE.KANJI
                if ln > 0:
                    original_text = text[i : i + ln]
                    kana_text = t
                    i += ln
                    output_flag = (False, False, False)
                else:  # unknown kanji
                    original_text = text[i]
                    kana_text = ""
                    i += 1
                    output_flag = (True, False, False)
            else:
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                _result.append(self._iconv.convert(text[i], ""))
                i += 1
                output_flag = (False, False, False)

            # Convert to kana and Output based on flag
            if output_flag[0] and output_flag[1]:
                original_text += text[i]
                kana_text += text[i]
                _result.append(self._iconv.convert(original_text, kana_text))
                original_text = ""
                kana_text = ""
                i += 1
            elif output_flag[0] and output_flag[2]:
                if len(original_text) > 0:
                    _result.append(self._iconv.convert(original_text, kana_text))
                original_text = text[i]
                kana_text = text[i]
                i += 1
            elif output_flag[2]:
                original_text += text[i]
                kana_text += text[i]
                i += 1
            else:
                pass

        # last word
        if len(original_text) > 0:
            _result.append(self._iconv.convert(original_text, kana_text))

        return _result



from typing import Any

class Configurations:
    data_path: Any = ...
    jisyo_hepburn_hira: str = ...
    jisyo_passport_hira: str = ...
    jisyo_kunrei_hira: str = ...
    jisyo_itaiji: str = ...
    jisyo_kanwa: str = ...
    jisyo_hepburn: str = ...
    jisyo_passport: str = ...
    jisyo_kunrei: str = ...
    jisyo_halfkana: str = ...
    def dictpath(self, dbfile: str) -> Any: ...

class Ch:
    space: int = ...
    at_mark: int = ...
    alphabet_A: int = ...
    alphabet_Z: int = ...
    square_bra: int = ...
    back_quote: int = ...
    alphabet_a: int = ...
    alphabet_z: int = ...
    bracket_bra: int = ...
    tilda: int = ...
    delete: int = ...
    ideographic_space: int = ...
    postal_mark_face: int = ...
    wavy_dash: int = ...
    ideographic_half_fill_space: int = ...
    greece_Alpha: int = ...
    greece_Rho: int = ...
    greece_Sigma: int = ...
    greece_Omega: int = ...
    greece_alpha: int = ...
    greece_omega: int = ...
    cyrillic_A: int = ...
    cyrillic_E: int = ...
    cyrillic_e: int = ...
    cyrillic_ya: int = ...
    zenkaku_exc_mark: int = ...
    zenkaku_slash_mark: int = ...
    zenkaku_number_zero: int = ...
    zenkaku_number_nine: int = ...
    zenkaku_A: int = ...
    zenkaku_a: int = ...
    endmark: str = ...

class Convert_Tables:
    symbol_table_1: Any = ...
    symbol_table_2: Any = ...
    symbol_table_3: Any = ...
    symbol_table_4: Any = ...
    symbol_table_5: Any = ...
    cyrillic_table: Any = ...
    alpha_table_1: Any = ...
    alpha_table_2: Any = ...
    alpha_table_3: Any = ...



# -*- coding: utf-8 -*-
import pytest

import pykakasi

original_text = (
    "日本国民は、正当に選挙された国会における代表者を通じて行動し、われらとわれらの子孫のために、"
    "諸国民との協和による成果と、わが国全土にわたつて自由のもたらす恵沢を確保し、"
    "政府の行為によつて再び戦争の惨禍が起ることのないやうにすることを決意し、ここに主権が国民に存することを宣言し、"
    "この憲法を確定する。そもそも国政は、国民の厳粛な信託によるものであつて、その権威は国民に由来し、"
    "その権力は国民の代表者がこれを行使し、その福利は国民がこれを享受する。これは人類普遍の原理であり、"
    "この憲法は、かかる原理に基くものである。われらは、これに反する一切の憲法、法令及び詔勅を排除する。"
)


expected = [
    {
        "orig": "日本国民",
        "kana": "ニホンコクミン",
        "hira": "にほんこくみん",
        "hepburn": "nihonkokumin",
        "kunrei": "nihonkokumin",
        "passport": "nihonkokumin",
    },
    {
        "orig": "は、",
        "kana": "ハ、",
        "hira": "は、",
        "hepburn": "ha,",
        "kunrei": "ha,",
        "passport": "ha,",
    },
    {
        "orig": "正当",
        "kana": "セイトウ",
        "hira": "せいとう",
        "hepburn": "seitou",
        "kunrei": "seitou",
        "passport": "seito",
    },
    {
        "orig": "に",
        "kana": "ニ",
        "hira": "に",
        "hepburn": "ni",
        "kunrei": "ni",
        "passport": "ni",
    },
    {
        "orig": "選挙",
        "kana": "センキョ",
        "hira": "せんきょ",
        "hepburn": "senkyo",
        "kunrei": "senkyo",
        "passport": "senkyo",
    },
    {
        "orig": "された",
        "kana": "サレタ",
        "hira": "された",
        "hepburn": "sareta",
        "kunrei": "sareta",
        "passport": "sareta",
    },
    {
        "orig": "国会",
        "kana": "コッカイ",
        "hira": "こっかい",
        "hepburn": "kokkai",
        "kunrei": "kokkai",
        "passport": "kokkai",
    },
    {
        "orig": "における",
        "kana": "ニオケル",
        "hira": "における",
        "hepburn": "niokeru",
        "kunrei": "niokeru",
        "passport": "niokeru",
    },
    {
        "orig": "代表者",
        "kana": "ダイヒョウシャ",
        "hira": "だいひょうしゃ",
        "hepburn": "daihyousha",
        "kunrei": "daihyousya",
        "passport": "daihyousha",
    },
    {
        "orig": "を",
        "kana": "ヲ",
        "hira": "を",
        "hepburn": "wo",
        "kunrei": "wo",
        "passport": "wo",
    },
    {
        "orig": "通じ",
        "kana": "ツウジ",
        "hira": "つうじ",
        "hepburn": "tsuuji",
        "kunrei": "tuuzi",
        "passport": "tsuuji",
    },
    {
        "orig": "て",
        "kana": "テ",
        "hira": "て",
        "hepburn": "te",
        "kunrei": "te",
        "passport": "te",
    },
    {
        "orig": "行動",
        "kana": "コウドウ",
        "hira": "こうどう",
        "hepburn": "koudou",
        "kunrei": "koudou",
        "passport": "kodou",
    },
    {
        "orig": "し、",
        "kana": "シ、",
        "hira": "し、",
        "hepburn": "shi,",
        "kunrei": "si,",
        "passport": "shi,",
    },
    {
        "orig": "われらとわれらの",
        "kana": "ワレラトワレラノ",
        "hira": "われらとわれらの",
        "hepburn": "wareratowarerano",
        "kunrei": "wareratowarerano",
        "passport": "wareratowarerano",
    },
    {
        "orig": "子孫",
        "kana": "シソン",
        "hira": "しそん",
        "hepburn": "shison",
        "kunrei": "sison",
        "passport": "shison",
    },
    {
        "orig": "のために、",
        "kana": "ノタメニ、",
        "hira": "のために、",
        "hepburn": "notameni,",
        "kunrei": "notameni,",
        "passport": "notameni,",
    },
    {
        "orig": "諸国民",
        "kana": "ショコクミン",
        "hira": "しょこくみん",
        "hepburn": "shokokumin",
        "kunrei": "syokokumin",
        "passport": "shokokumin",
    },
    {
        "orig": "との",
        "kana": "トノ",
        "hira": "との",
        "hepburn": "tono",
        "kunrei": "tono",
        "passport": "tono",
    },
    {
        "orig": "協和",
        "kana": "キョウワ",
        "hira": "きょうわ",
        "hepburn": "kyouwa",
        "kunrei": "kyouwa",
        "passport": "kyouwa",
    },
    {
        "orig": "による",
        "kana": "ニヨル",
        "hira": "による",
        "hepburn": "niyoru",
        "kunrei": "niyoru",
        "passport": "niyoru",
    },
    {
        "orig": "成果",
        "kana": "セイカ",
        "hira": "せいか",
        "hepburn": "seika",
        "kunrei": "seika",
        "passport": "seika",
    },
    {
        "orig": "と、",
        "kana": "ト、",
        "hira": "と、",
        "hepburn": "to,",
        "kunrei": "to,",
        "passport": "to,",
    },
    {
        "orig": "わが",
        "kana": "ワガ",
        "hira": "わが",
        "hepburn": "waga",
        "kunrei": "waga",
        "passport": "waga",
    },
    {
        "orig": "国",
        "kana": "クニ",
        "hira": "くに",
        "hepburn": "kuni",
        "kunrei": "kuni",
        "passport": "kuni",
    },
    {
        "orig": "全土",
        "kana": "ゼンド",
        "hira": "ぜんど",
        "hepburn": "zendo",
        "kunrei": "zendo",
        "passport": "zendo",
    },
    {
        "orig": "にわたつて",
        "kana": "ニワタツテ",
        "hira": "にわたつて",
        "hepburn": "niwatatsute",
        "kunrei": "niwatatute",
        "passport": "niwatatsute",
    },
    {
        "orig": "自由",
        "kana": "ジユウ",
        "hira": "じゆう",
        "hepburn": "jiyuu",
        "kunrei": "ziyuu",
        "passport": "jiyuu",
    },
    {
        "orig": "のもたらす",
        "kana": "ノモタラス",
        "hira": "のもたらす",
        "hepburn": "nomotarasu",
        "kunrei": "nomotarasu",
        "passport": "nomotarasu",
    },
    {
        "orig": "恵沢",
        "kana": "ケイタク",
        "hira": "けいたく",
        "hepburn": "keitaku",
        "kunrei": "keitaku",
        "passport": "keitaku",
    },
    {
        "orig": "を",
        "kana": "ヲ",
        "hira": "を",
        "hepburn": "wo",
        "kunrei": "wo",
        "passport": "wo",
    },
    {
        "orig": "確保",
        "kana": "カクホ",
        "hira": "かくほ",
        "hepburn": "kakuho",
        "kunrei": "kakuho",
        "passport": "kakuho",
    },
    {
        "orig": "し、",
        "kana": "シ、",
        "hira": "し、",
        "hepburn": "shi,",
        "kunrei": "si,",
        "passport": "shi,",
    },
    {
        "orig": "政府",
        "kana": "セイフ",
        "hira": "せいふ",
        "hepburn": "seifu",
        "kunrei": "seifu",
        "passport": "seifu",
    },
    {
        "orig": "の",
        "kana": "ノ",
        "hira": "の",
        "hepburn": "no",
        "kunrei": "no",
        "passport": "no",
    },
    {
        "orig": "行為",
        "kana": "コウイ",
        "hira": "こうい",
        "hepburn": "koui",
        "kunrei": "koui",
        "passport": "koi",
    },
    {
        "orig": "によつて",
        "kana": "ニヨツテ",
        "hira": "によつて",
        "hepburn": "niyotsute",
        "kunrei": "niyotute",
        "passport": "niyotsute",
    },
    {
        "orig": "再び",
        "kana": "フタタビ",
        "hira": "ふたたび",
        "hepburn": "futatabi",
        "kunrei": "futatabi",
        "passport": "futatabi",
    },
    {
        "orig": "戦争",
        "kana": "センソウ",
        "hira": "せんそう",
        "hepburn": "sensou",
        "kunrei": "sensou",
        "passport": "senso",
    },
    {
        "orig": "の",
        "kana": "ノ",
        "hira": "の",
        "hepburn": "no",
        "kunrei": "no",
        "passport": "no",
    },
    {
        "orig": "惨禍",
        "kana": "サンカ",
        "hira": "さんか",
        "hepburn": "sanka",
        "kunrei": "sanka",
        "passport": "sanka",
    },
    {
        "orig": "が",
        "kana": "ガ",
        "hira": "が",
        "hepburn": "ga",
        "kunrei": "ga",
        "passport": "ga",
    },
    {
        "orig": "起る",
        "kana": "オコル",
        "hira": "おこる",
        "hepburn": "okoru",
        "kunrei": "okoru",
        "passport": "okoru",
    },
    {
        "orig": "ことのないやうにすることを",
        "kana": "コトノナイヤウニスルコトヲ",
        "hira": "ことのないやうにすることを",
        "hepburn": "kotononaiyaunisurukotowo",
        "kunrei": "kotononaiyaunisurukotowo",
        "passport": "kotononaiyaunisurukotowo",
    },
    {
        "orig": "決意",
        "kana": "ケツイ",
        "hira": "けつい",
        "hepburn": "ketsui",
        "kunrei": "ketui",
        "passport": "ketsui",
    },
    {
        "orig": "し、",
        "kana": "シ、",
        "hira": "し、",
        "hepburn": "shi,",
        "kunrei": "si,",
        "passport": "shi,",
    },
    {
        "orig": "ここに",
        "kana": "ココニ",
        "hira": "ここに",
        "hepburn": "kokoni",
        "kunrei": "kokoni",
        "passport": "kokoni",
    },
    {
        "orig": "主権",
        "kana": "シュケン",
        "hira": "しゅけん",
        "hepburn": "shuken",
        "kunrei": "syuken",
        "passport": "shuken",
    },
    {
        "orig": "が",
        "kana": "ガ",
        "hira": "が",
        "hepburn": "ga",
        "kunrei": "ga",
        "passport": "ga",
    },
    {
        "orig": "国民",
        "kana": "コクミン",
        "hira": "こくみん",
        "hepburn": "kokumin",
        "kunrei": "kokumin",
        "passport": "kokumin",
    },
    {
        "orig": "に",
        "kana": "ニ",
        "hira": "に",
        "hepburn": "ni",
        "kunrei": "ni",
        "passport": "ni",
    },
    {
        "orig": "存す",
        "kana": "ソンス",
        "hira": "そんす",
        "hepburn": "sonsu",
        "kunrei": "sonsu",
        "passport": "sonsu",
    },
    {
        "orig": "ることを",
        "kana": "ルコトヲ",
        "hira": "ることを",
        "hepburn": "rukotowo",
        "kunrei": "rukotowo",
        "passport": "rukotowo",
    },
    {
        "orig": "宣言",
        "kana": "センゲン",
        "hira": "せんげん",
        "hepburn": "sengen",
        "kunrei": "sengen",
        "passport": "sengen",
    },
    {
        "orig": "し、",
        "kana": "シ、",
        "hira": "し、",
        "hepburn": "shi,",
        "kunrei": "si,",
        "passport": "shi,",
    },
    {
        "orig": "この",
        "kana": "コノ",
        "hira": "この",
        "hepburn": "kono",
        "kunrei": "kono",
        "passport": "kono",
    },
    {
        "orig": "憲法",
        "kana": "ケンポウ",
        "hira": "けんぽう",
        "hepburn": "kenpou",
        "kunrei": "kenpou",
        "passport": "kempou",
    },
    {
        "orig": "を",
        "kana": "ヲ",
        "hira": "を",
        "hepburn": "wo",
        "kunrei": "wo",
        "passport": "wo",
    },
    {
        "orig": "確定す",
        "kana": "カクテイス",
        "hira": "かくていす",
        "hepburn": "kakuteisu",
        "kunrei": "kakuteisu",
        "passport": "kakuteisu",
    },
    {
        "orig": "る。",
        "kana": "ル。",
        "hira": "る。",
        "hepburn": "ru.",
        "kunrei": "ru.",
        "passport": "ru.",
    },
    {
        "orig": "そもそも",
        "kana": "ソモソモ",
        "hira": "そもそも",
        "hepburn": "somosomo",
        "kunrei": "somosomo",
        "passport": "somosomo",
    },
    {
        "orig": "国政",
        "kana": "コクセイ",
        "hira": "こくせい",
        "hepburn": "kokusei",
        "kunrei": "kokusei",
        "passport": "kokusei",
    },
    {
        "orig": "は、",
        "kana": "ハ、",
        "hira": "は、",
        "hepburn": "ha,",
        "kunrei": "ha,",
        "passport": "ha,",
    },
    {
        "orig": "国民",
        "kana": "コクミン",
        "hira": "こくみん",
        "hepburn": "kokumin",
        "kunrei": "kokumin",
        "passport": "kokumin",
    },
    {
        "orig": "の",
        "kana": "ノ",
        "hira": "の",
        "hepburn": "no",
        "kunrei": "no",
        "passport": "no",
    },
    {
        "orig": "厳粛",
        "kana": "ゲンシュク",
        "hira": "げんしゅく",
        "hepburn": "genshuku",
        "kunrei": "gensyuku",
        "passport": "genshuku",
    },
    {
        "orig": "な",
        "kana": "ナ",
        "hira": "な",
        "hepburn": "na",
        "kunrei": "na",
        "passport": "na",
    },
    {
        "orig": "信託",
        "kana": "シンタク",
        "hira": "しんたく",
        "hepburn": "shintaku",
        "kunrei": "sintaku",
        "passport": "shintaku",
    },
    {
        "orig": "によるものであつて、",
        "kana": "ニヨルモノデアツテ、",
        "hira": "によるものであつて、",
        "hepburn": "niyorumonodeatsute,",
        "kunrei": "niyorumonodeatute,",
        "passport": "niyorumonodeatsute,",
    },
    {
        "orig": "その",
        "kana": "ソノ",
        "hira": "その",
        "hepburn": "sono",
        "kunrei": "sono",
        "passport": "sono",
    },
    {
        "orig": "権威",
        "kana": "ケンイ",
        "hira": "けんい",
        "hepburn": "ken'i",
        "kunrei": "ken'i",
        "passport": "keni",
    },
    {
        "orig": "は",
        "kana": "ハ",
        "hira": "は",
        "hepburn": "ha",
        "kunrei": "ha",
        "passport": "ha",
    },
    {
        "orig": "国民",
        "kana": "コクミン",
        "hira": "こくみん",
        "hepburn": "kokumin",
        "kunrei": "kokumin",
        "passport": "kokumin",
    },
    {
        "orig": "に",
        "kana": "ニ",
        "hira": "に",
        "hepburn": "ni",
        "kunrei": "ni",
        "passport": "ni",
    },
    {
        "orig": "由来",
        "kana": "ユライ",
        "hira": "ゆらい",
        "hepburn": "yurai",
        "kunrei": "yurai",
        "passport": "yurai",
    },
    {
        "orig": "し、",
        "kana": "シ、",
        "hira": "し、",
        "hepburn": "shi,",
        "kunrei": "si,",
        "passport": "shi,",
    },
    {
        "orig": "その",
        "kana": "ソノ",
        "hira": "その",
        "hepburn": "sono",
        "kunrei": "sono",
        "passport": "sono",
    },
    {
        "orig": "権力",
        "kana": "ケンリョク",
        "hira": "けんりょく",
        "hepburn": "kenryoku",
        "kunrei": "kenryoku",
        "passport": "kenryoku",
    },
    {
        "orig": "は",
        "kana": "ハ",
        "hira": "は",
        "hepburn": "ha",
        "kunrei": "ha",
        "passport": "ha",
    },
    {
        "orig": "国民",
        "kana": "コクミン",
        "hira": "こくみん",
        "hepburn": "kokumin",
        "kunrei": "kokumin",
        "passport": "kokumin",
    },
    {
        "orig": "の",
        "kana": "ノ",
        "hira": "の",
        "hepburn": "no",
        "kunrei": "no",
        "passport": "no",
    },
    {
        "orig": "代表者",
        "kana": "ダイヒョウシャ",
        "hira": "だいひょうしゃ",
        "hepburn": "daihyousha",
        "kunrei": "daihyousya",
        "passport": "daihyousha",
    },
    {
        "orig": "がこれを",
        "kana": "ガコレヲ",
        "hira": "がこれを",
        "hepburn": "gakorewo",
        "kunrei": "gakorewo",
        "passport": "gakorewo",
    },
    {
        "orig": "行使",
        "kana": "コウシ",
        "hira": "こうし",
        "hepburn": "koushi",
        "kunrei": "kousi",
        "passport": "koshi",
    },
    {
        "orig": "し、",
        "kana": "シ、",
        "hira": "し、",
        "hepburn": "shi,",
        "kunrei": "si,",
        "passport": "shi,",
    },
    {
        "orig": "その",
        "kana": "ソノ",
        "hira": "その",
        "hepburn": "sono",
        "kunrei": "sono",
        "passport": "sono",
    },
    {
        "orig": "福利",
        "kana": "フクリ",
        "hira": "ふくり",
        "hepburn": "fukuri",
        "kunrei": "fukuri",
        "passport": "fukuri",
    },
    {
        "orig": "は",
        "kana": "ハ",
        "hira": "は",
        "hepburn": "ha",
        "kunrei": "ha",
        "passport": "ha",
    },
    {
        "orig": "国民",
        "kana": "コクミン",
        "hira": "こくみん",
        "hepburn": "kokumin",
        "kunrei": "kokumin",
        "passport": "kokumin",
    },
    {
        "orig": "がこれを",
        "kana": "ガコレヲ",
        "hira": "がこれを",
        "hepburn": "gakorewo",
        "kunrei": "gakorewo",
        "passport": "gakorewo",
    },
    {
        "orig": "享受",
        "kana": "キョウジュ",
        "hira": "きょうじゅ",
        "hepburn": "kyouju",
        "kunrei": "kyouju",
        "passport": "kyouju",
    },
    {
        "orig": "する。",
        "kana": "スル。",
        "hira": "する。",
        "hepburn": "suru.",
        "kunrei": "suru.",
        "passport": "suru.",
    },
    {
        "orig": "これは",
        "kana": "コレハ",
        "hira": "これは",
        "hepburn": "koreha",
        "kunrei": "koreha",
        "passport": "koreha",
    },
    {
        "orig": "人類普遍",
        "kana": "ジンルイフヘン",
        "hira": "じんるいふへん",
        "hepburn": "jinruifuhen",
        "kunrei": "zinruifuhen",
        "passport": "jinruifuhen",
    },
    {
        "orig": "の",
        "kana": "ノ",
        "hira": "の",
        "hepburn": "no",
        "kunrei": "no",
        "passport": "no",
    },
    {
        "orig": "原理",
        "kana": "ゲンリ",
        "hira": "げんり",
        "hepburn": "genri",
        "kunrei": "genri",
        "passport": "genri",
    },
    {
        "orig": "であり、",
        "kana": "デアリ、",
        "hira": "であり、",
        "hepburn": "deari,",
        "kunrei": "deari,",
        "passport": "deari,",
    },
    {
        "orig": "この",
        "kana": "コノ",
        "hira": "この",
        "hepburn": "kono",
        "kunrei": "kono",
        "passport": "kono",
    },
    {
        "orig": "憲法",
        "kana": "ケンポウ",
        "hira": "けんぽう",
        "hepburn": "kenpou",
        "kunrei": "kenpou",
        "passport": "kempou",
    },
    {
        "orig": "は、",
        "kana": "ハ、",
        "hira": "は、",
        "hepburn": "ha,",
        "kunrei": "ha,",
        "passport": "ha,",
    },
    {
        "orig": "かかる",
        "kana": "カカル",
        "hira": "かかる",
        "hepburn": "kakaru",
        "kunrei": "kakaru",
        "passport": "kakaru",
    },
    {
        "orig": "原理",
        "kana": "ゲンリ",
        "hira": "げんり",
        "hepburn": "genri",
        "kunrei": "genri",
        "passport": "genri",
    },
    {
        "orig": "に",
        "kana": "ニ",
        "hira": "に",
        "hepburn": "ni",
        "kunrei": "ni",
        "passport": "ni",
    },
    {
        "orig": "基く",
        "kana": "モトヅク",
        "hira": "もとづく",
        "hepburn": "motozuku",
        "kunrei": "motozuku",
        "passport": "motozuku",
    },
    {
        "orig": "ものである。",
        "kana": "モノデアル。",
        "hira": "ものである。",
        "hepburn": "monodearu.",
        "kunrei": "monodearu.",
        "passport": "monodearu.",
    },
    {
        "orig": "われらは、",
        "kana": "ワレラハ、",
        "hira": "われらは、",
        "hepburn": "wareraha,",
        "kunrei": "wareraha,",
        "passport": "wareraha,",
    },
    {
        "orig": "これに",
        "kana": "コレニ",
        "hira": "これに",
        "hepburn": "koreni",
        "kunrei": "koreni",
        "passport": "koreni",
    },
    {
        "orig": "反す",
        "kana": "ハンス",
        "hira": "はんす",
        "hepburn": "hansu",
        "kunrei": "hansu",
        "passport": "hansu",
    },
    {
        "orig": "る",
        "kana": "ル",
        "hira": "る",
        "hepburn": "ru",
        "kunrei": "ru",
        "passport": "ru",
    },
    {
        "orig": "一切",
        "kana": "イッサイ",
        "hira": "いっさい",
        "hepburn": "issai",
        "kunrei": "issai",
        "passport": "issai",
    },
    {
        "orig": "の",
        "kana": "ノ",
        "hira": "の",
        "hepburn": "no",
        "kunrei": "no",
        "passport": "no",
    },
    {
        "orig": "憲法、",
        "kana": "ケンポウ、",
        "hira": "けんぽう、",
        "hepburn": "kenpou,",
        "kunrei": "kenpou,",
        "passport": "kempou,",
    },
    {
        "orig": "法令",
        "kana": "ホウレイ",
        "hira": "ほうれい",
        "hepburn": "hourei",
        "kunrei": "hourei",
        "passport": "hourei",
    },
    {
        "orig": "及び",
        "kana": "オヨビ",
        "hira": "および",
        "hepburn": "oyobi",
        "kunrei": "oyobi",
        "passport": "oyobi",
    },
    {
        "orig": "詔勅",
        "kana": "ショウチョク",
        "hira": "しょうちょく",
        "hepburn": "shouchoku",
        "kunrei": "syoutyoku",
        "passport": "shouchoku",
    },
    {
        "orig": "を",
        "kana": "ヲ",
        "hira": "を",
        "hepburn": "wo",
        "kunrei": "wo",
        "passport": "wo",
    },
    {
        "orig": "排除",
        "kana": "ハイジョ",
        "hira": "はいじょ",
        "hepburn": "haijo",
        "kunrei": "haijo",
        "passport": "haijo",
    },
    {
        "orig": "する。",
        "kana": "スル。",
        "hira": "する。",
        "hepburn": "suru.",
        "kunrei": "suru.",
        "passport": "suru.",
    },
]


def test_kakasi_structured_constitution():
    kakasi = pykakasi.kakasi()
    result = kakasi.convert(original_text)
    for i, e in enumerate(expected):
        assert result[i]["orig"] == e["orig"]
        assert result[i]["hira"] == e["hira"]
        assert result[i]["kana"] == e["kana"]
        assert result[i]["hepburn"] == e["hepburn"]
        assert result[i]["kunrei"] == e["kunrei"]
        assert result[i]["passport"] == e["passport"]


@pytest.mark.benchmark
def test_benchmark(benchmark):
    kakasi = pykakasi.kakasi()
    benchmark.extra_info["data_size"] = len(original_text)
    result = benchmark(kakasi.convert, original_text)
    assert result[0]["hira"] == expected[0]["hira"]



# the upstream session-scoped test fixture redundantly re-generates
# the dictionaries that are already packaged, see
# https://github.com/miurahr/pykakasi/blob/v2.0.4/tests/conftest.py
# unless we specify the following environment variable:
import os
os.environ["TOX_ENV_DIR"] = "dummy"

import pytest
pytest.main(["tests/"])



# -*- coding: utf-8 -*-
# scripts.py
#
# Copyright 2011-2019 Hiroshi Miura <miurahr@linux.com>
import functools
import pickle
from typing import Dict

from .properties import Ch, Configurations, Convert_Tables


class IConv:

    _MAXLEN: int = 32

    def __init__(self):
        self._hahconv = H2("a", method="Hepburn")
        self._hakconv = H2("a", method="Kunrei")
        self._hapconv = H2("a", method="Passport")
        self._hkconv = H2("K")
        self._khconv = K2("H")
        self._saconv = Sym2("a")

    @functools.lru_cache(maxsize=256)
    def convert(self, otext: str, hira: str) -> Dict[str, str]:
        kana = self._h2k(hira)
        hira = self._k2h(hira)  # make sure hiragana doesn't contain katakana
        tmp = {
            "orig": otext,
            "hira": hira,
            "kana": kana,
            "hepburn": self._s2a(self._h2ah(hira)),
            "kunrei": self._s2a(self._h2ak(hira)),
            "passport": self._s2a(self._h2ap(hira)),
        }
        return tmp

    def _s2a(self, text: str) -> str:
        result = ""  # type: str
        i = 0
        length = len(text)
        while i < length:
            w = min(i + self._MAXLEN, length)  # type: int
            (t, l1) = self._saconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            elif text[i] in Ch.long_symbols:  # handle chōonpu sound marks
                # use previous char as a transliteration for kana-dash
                if len(result) > 0:
                    result += result[-1]
                else:
                    result += "-"
                i += 1
            else:
                result += text[i : i + 1]
                i += 1
        return result

    def _k2h(self, text: str) -> str:
        result = ""
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._khconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i : i + 1]
                i += 1
        return result

    def _h2k(self, text: str) -> str:
        result = ""
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hkconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i : i + 1]
                i += 1
        return result

    def _h2ak(self, text: str) -> str:
        result = ""
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hakconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i : i + 1]
                i += 1
        return result

    def _h2ah(self, text: str) -> str:
        result = ""
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hahconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i : i + 1]
                i += 1
        return result

    def _h2ap(self, text: str) -> str:
        result = ""
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hapconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i : i + 1]
                i += 1
        return result


class H2:

    _kanadict = None

    _diff = 0x30A1 - 0x3041  # KATAKANA LETTER A - HIRAGANA A
    _ediff = 0x1B164 - 0x1B150

    def __init__(self, mode, method="Hepburn"):
        if mode == "a":
            if method == "Passport":
                self._kanadict = Jisyo(Configurations.jisyo_passport_hira)
            elif method == "Kunrei":
                self._kanadict = Jisyo(Configurations.jisyo_kunrei_hira)
            else:
                self._kanadict = Jisyo(Configurations.jisyo_hepburn_hira)

            self.convert = self.convert_a
        elif mode == "K":
            self.convert = self.convert_K
        else:
            self.convert = self.convert_noop

    @classmethod
    def isRegion(cls, char):
        return 0x3040 < ord(char[0]) < 0x3097 or 0x1B150 <= ord(char[0]) <= 0x1B152

    def convert_a(self, text):
        Hstr = ""
        max_len = -1
        r = min(self._kanadict.maxkeylen(), len(text))
        for x in range(1, r + 1):
            if self._kanadict.haskey(text[:x]):
                if max_len < x:
                    max_len = x
                    Hstr = self._kanadict.lookup(text[:x])
        return (Hstr, max_len)

    def convert_K(self, text):
        Hstr = ""
        max_len = 0
        r = len(text)
        for x in range(r):
            if 0x3040 < ord(text[x]) < 0x3097:
                Hstr = Hstr + chr(ord(text[x]) + self._diff)
                max_len += 1
            elif 0x1B150 <= ord(text[x]) <= 0x1B152:
                Hstr = Hstr + chr(ord(text[x]) + self._ediff)
                max_len += 1
            else:  # pragma: no cover
                break
        return (Hstr, max_len)

    def convert_noop(self, text):
        return (text[0], 1)


class K2:

    _kanadict = None
    _halfkanadict = None

    _diff = 0x30A1 - 0x3041  # KATAKANA LETTER A - HIRAGANA A
    _ediff = 0x1B164 - 0x1B150

    def __init__(self, mode, method="Hepburn"):
        self._halfkanadict = Jisyo(Configurations.jisyo_halfkana)
        if mode == "a":
            if method == "Passport":
                self._kanadict = Jisyo(Configurations.jisyo_passport)
            elif method == "Kunrei":
                self._kanadict = Jisyo(Configurations.jisyo_kunrei)
            else:
                self._kanadict = Jisyo(Configurations.jisyo_hepburn)

            self.convert = self.convert_a
        elif mode == "H":
            self.convert = self.convert_h
        else:
            self.convert = self.convert_noop

    @classmethod
    def isRegion(cls, char):
        ch = ord(char[0])
        return (
            cls._is_katakana(ch)
            or cls._is_half_width_kana(ch)
            or 0x1B164 <= ch <= 0x1B167
        )

    @classmethod
    def _is_katakana(cls, ch):
        return 0x30A0 < ch < 0x30FD

    @classmethod
    def _is_half_width_kana(cls, ch):
        return 0xFF65 < ch < 0xFF9F

    def _convert_half_kana(self, text):
        Hstr = ""
        max_len = -1
        for x in [2, 1]:
            if self._halfkanadict.haskey(text[:x]):
                max_len = x
                Hstr = self._halfkanadict.lookup(text[:x])
                break
        return Hstr, max_len

    def convert_a(self, text):
        Hstr = ""
        max_len = -1
        r = min(self._kanadict.maxkeylen(), len(text))
        for x in range(1, r + 1):
            if self._kanadict.haskey(text[:x]):
                if max_len < x:
                    max_len = x
                    Hstr = self._kanadict.lookup(text[:x])
        return Hstr, max_len

    def convert_h(self, text):
        Hstr = ""
        max_len = 0
        r = len(text)
        x = 0
        while x < r:
            if 0x1B164 <= ord(text[x]) < 0x1B167:
                Hstr = Hstr + chr(ord(text[x]) - self._ediff)
                max_len += 1
                x += 1
            elif ord(text[x]) == 0x1B167:
                Hstr = Hstr + "\u3093"
                max_len += 1
                x += 1
            elif 0x30A0 < ord(text[x]) < 0x30F7:
                Hstr = Hstr + chr(ord(text[x]) - self._diff)
                max_len += 1
                x += 1
            elif 0x30F7 <= ord(text[x]) < 0x30FD:
                Hstr = Hstr + text[x]
                max_len += 1
                x += 1
            elif self._is_half_width_kana(ord(text[x])):
                kstr, length = self._convert_half_kana(text[x:])
                if length > 0:
                    max_len += length
                    x += length
                    if ord(kstr) == 0x309B:
                        Hstr = Hstr + kstr
                    else:
                        Hstr = Hstr + chr(ord(kstr) - self._diff)
                else:
                    max_len += 1
                    x += 1  # skip unknown character(issue #115)
            else:  # pragma: no cover
                break
        return (Hstr, max_len)

    def convert_noop(self, text):
        return text[0], 1


class Jisyo:
    _dict = None

    def __init__(self, dictname):
        src = Configurations.dictpath(dictname)
        with open(src, "rb") as d:
            self._dict = pickle.load(d)

    def haskey(self, key):
        return key in self._dict

    def lookup(self, key):
        return self._dict[key]

    def maxkeylen(self):
        return self._dict["_max_key_len_"]


class Sym2:
    def __init__(self, mode):
        if mode == "a":
            self.convert = self.convert_a
        else:
            self.convert = self.convert_noop

    @classmethod
    def isRegion(cls, char: str):
        c = ord(char[0])
        return (
            (Ch.ideographic_space <= c <= Ch.postal_mark_face)
            or (Ch.wavy_dash <= c <= Ch.ideographic_half_fill_space)
            or (Ch.greece_Alpha <= c <= Ch.greece_Rho)
            or (Ch.greece_Sigma <= c <= Ch.greece_Omega)
            or (Ch.greece_alpha <= c <= Ch.greece_omega)
            or (Ch.cyrillic_A <= c <= Ch.cyrillic_ya)
            or (Ch.zenkaku_exc_mark <= c <= Ch.zenkaku_number_nine)
            or (0xFF20 <= c <= 0xFF5E)
            or c == 0x0451
            or c == 0x0401
        )

    def _convert(self, text):
        c = ord(text[0])
        if Ch.ideographic_space <= c <= Ch.postal_mark_face:
            return Convert_Tables.symbol_table_1[c - Ch.ideographic_space]
        elif Ch.wavy_dash <= c <= Ch.ideographic_half_fill_space:
            return Convert_Tables.symbol_table_2[c - Ch.wavy_dash]
        elif Ch.greece_Alpha <= c <= Ch.greece_Omega:
            return Convert_Tables.symbol_table_3[c - Ch.greece_Alpha]
        elif Ch.greece_alpha <= c <= Ch.greece_omega:
            return Convert_Tables.symbol_table_4[c - Ch.greece_alpha]
        elif Ch.cyrillic_A <= c <= Ch.cyrillic_ya:
            return Convert_Tables.cyrillic_table[text[0]]
        elif c == Ch.cyrillic_E or c == Ch.cyrillic_e:
            return Convert_Tables.cyrillic_table[text[0]]
        elif Ch.zenkaku_exc_mark <= c <= Ch.zenkaku_slash_mark:
            return Convert_Tables.symbol_table_5[c - Ch.zenkaku_exc_mark]
        elif Ch.zenkaku_number_zero <= c <= Ch.zenkaku_number_nine:
            return chr(c - Ch.zenkaku_number_zero + ord("0"))
        elif 0xFF20 <= c <= 0xFF40:
            return chr(0x0041 + c - 0xFF21)  # u\ff21Ａ => u\0041:@A..Z[\]^_`
        elif 0xFF41 <= c < 0xFF5F:
            return chr(0x0061 + c - 0xFF41)  # u\ff41ａ => u\0061:a..z{|}
        else:
            return ""  # pragma: no cover

    def convert_a(self, text):
        t = self._convert(text)
        if t is not None and len(t) > 0:
            return t, 1
        else:
            return "", 0

    def convert_noop(self, text):
        return text[0], 1


class A2:
    def __init__(self, mode):
        if mode == "E":
            self.convert = self.convert_E
        else:
            self.convert = self.convert_noop

    @classmethod
    def isRegion(cls, char):
        return Ch.space <= ord(char[0]) < Ch.delete

    def _convert(self, text):
        c = ord(text[0])
        if Ch.space <= c <= Ch.at_mark:
            return Convert_Tables.alpha_table_1[(c - Ch.space)]
        elif Ch.alphabet_A <= c <= Ch.alphabet_Z:
            return chr(Ch.zenkaku_A + c - Ch.alphabet_A)  # u\0041A => u\ff21Ａ
        elif Ch.square_bra <= c <= Ch.back_quote:
            return Convert_Tables.alpha_table_2[(c - Ch.square_bra)]
        elif Ch.alphabet_a <= c <= Ch.alphabet_z:
            return chr(Ch.zenkaku_a + c - Ch.alphabet_a)  # u\0061a => u\ff41ａ
        elif Ch.bracket_bra <= c <= Ch.tilda:
            return Convert_Tables.alpha_table_3[(c - Ch.bracket_bra)]
        else:
            return ""  # pragma: no cover

    def convert_E(self, text):
        t = self._convert(text)
        if len(t):
            return t, 1
        else:
            return "", 0

    def convert_noop(self, text):
        return text[0], 1



# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011-2021 Hiroshi Miura <miurahr@linux.com>
from typing import Dict, List, Optional, Tuple, Union

from deprecated import deprecated  # type: ignore  # noqa

from .kakasi import Kakasi, PyKakasiException
from .kanji import Itaiji, JConv
from .properties import Ch
from .scripts import A2, H2, K2, Sym2


class UnsupportedRomanRulesException(PyKakasiException):
    pass


class UnknownOptionsException(PyKakasiException):
    pass


class InvalidModeValueException(PyKakasiException):
    pass


class InvalidFlagValueException(PyKakasiException):
    pass


class kakasi:

    _keys: List[str] = ["J", "H", "K", "E", "a"]
    _values: List[str] = ["a", "E", "H", "K"]
    _roman_vals: List[str] = ["Hepburn", "Kunrei", "Passport"]

    def __init__(self):
        self._kakasi = Kakasi()
        # for v1 api
        self._conv = {}  # type: Dict[str, Union[J2, H2, K2, A2, Sym2]]
        self._mode = {
            "J": None,
            "H": None,
            "K": None,
            "E": None,
            "a": None,
        }  # type: Dict[str, Optional[str]]
        self._furi = {
            "J": False,
            "H": False,
            "K": False,
            "E": False,
            "a": False,
        }  # type: Dict[str, bool]
        self._flag = {
            "p": False,
            "s": False,
            "f": False,
            "c": False,
            "C": False,
            "U": False,
            "u": False,
            "t": True,
        }  # type: Dict[str, bool]
        self._option = {"r": "Hepburn"}  # type: Dict[str, str]
        self._separator = " "  # type: str
        self._separator_string = " "  # type: str

    # v2 API
    def convert(self, text: str) -> List[Dict[str, str]]:
        return self._kakasi.convert(text)

    # v1 API implementations
    @deprecated(version=2.1, reason="Old API will be removed in v3.0.")
    def setMode(self, fr: str, to: Optional[Union[bool, str]]) -> None:
        if fr in self._keys:
            if to is None:
                self._mode[fr] = None
            elif isinstance(to, str) and to[0] in self._values:
                self._mode[fr] = to[0]
                if len(to) == 2 and to[1] == "F":
                    self._furi[fr] = True
            else:
                raise InvalidModeValueException("Invalid value for mode")
        elif fr in self._flag.keys():
            if isinstance(to, bool):
                self._flag[fr] = to
            else:
                raise InvalidFlagValueException("Invalid flag value")
        elif fr == "r":
            if isinstance(to, str) and to in self._roman_vals:
                self._option["r"] = to
            else:
                raise UnsupportedRomanRulesException("Unknown roman table name")
        elif fr == "S":
            if isinstance(to, str):
                self._separator = to
            else:
                raise InvalidFlagValueException("Incompatible separator value")
        else:
            raise UnknownOptionsException("Unhandled options")  # pragma: no cover

    @deprecated(version=2.1, reason="Old API will be removed in v3.0.")
    def getConverter(self):
        self._conv["J"] = J2(self._mode["J"], method=self._option["r"])
        self._conv["H"] = H2(self._mode["H"], method=self._option["r"])
        self._conv["K"] = K2(self._mode["K"], method=self._option["r"])
        self._conv["E"] = Sym2(self._mode["E"])
        self._conv["a"] = A2(self._mode["a"])
        return self

    @deprecated(version=2.1, reason="Old API will be removed in v3.0.")
    def do(self, text: str) -> str:
        _MAXLEN = 32
        otext = ""
        i = 0
        while True:
            if i >= len(text):
                break

            if self._conv["J"].isRegion(text[i]):
                mode = "J"
            elif self._conv["H"].isRegion(text[i]):
                mode = "H"
            elif self._conv["K"].isRegion(text[i]):
                mode = "K"
            elif self._conv["E"].isRegion(text[i]):
                mode = "E"
            elif self._conv["a"].isRegion(text[i]):
                mode = "a"
            else:
                mode = "o"

            if mode in ("J", "E"):
                w = min(i + _MAXLEN, len(text))
                (t, l1) = self._conv[mode].convert(text[i:w])

                if l1 > 0:
                    orig = text[i : i + l1]
                    chunk = t
                    i += l1
                else:
                    orig = text[i : i + 1]
                    if self._flag["t"]:
                        chunk = orig
                    else:
                        chunk = "???"
                    i += 1

            elif mode in ("H", "K", "a"):
                orig = ""
                chunk = ""

                while i < len(text):

                    if text[i] in "\u30FC\u2015\u2212\uFF70":

                        # FIXME: q&d workaround when hiragana/katanaka dash is first char.
                        if self._mode[mode] is not None and len(chunk) > 0:
                            # use previous char as a transliteration for kana-dash
                            orig += text[i]
                            chunk = chunk + chunk[-1]
                            i += 1
                        elif len(chunk) == 0:
                            orig += text[i]
                            chunk += "-"
                            i += 1
                            break
                        else:
                            orig += text[i]
                            chunk += text[i]
                            i += 1
                            break

                    elif self._conv[mode].isRegion(text[i]):
                        w = min(i + _MAXLEN, len(text))
                        (t, l1) = self._conv[mode].convert(text[i:w])
                        if l1 > 0:
                            orig += text[i : i + l1]
                            chunk += t
                            i += l1
                        else:
                            orig = text[i : i + 1]
                            if self._flag["t"]:
                                chunk = orig
                            else:
                                chunk = "???"
                            i += 1
                            break

                    else:
                        # i += 1
                        break

            else:
                otext += text[i]
                i += 1
                continue

            if mode in ("J", "E"):
                if self._flag["U"]:
                    chunk = chunk.upper()
                elif self._flag["C"]:
                    chunk = chunk.capitalize()

            if mode in self._keys and self._furi[mode]:
                otext += orig + "[" + chunk + "]"
            else:
                otext += chunk

            # insert separator when option specified and it is not a last character and not an end mark
            if (
                self._flag["s"]
                and otext[-len(self._separator) :] != self._separator
                and i < len(text)
                and text[i] not in Ch.endmark
            ):
                otext += self._separator

        return otext


@deprecated(version=2.1, reason="Old API will be removed in v3.0.")
class wakati(kakasi):
    def __init__(self):
        super(wakati, self).__init__()
        self._jconv = JConv()
        self._state = True  # type: bool

    def getConverter(self):
        return self

    def setMode(self, fr: str, to: Optional[Union[bool, str]]) -> None:
        if fr in self._flag.keys():
            if isinstance(to, bool):
                self._flag[fr] = to
            else:
                raise InvalidFlagValueException("Invalid flag value")
            raise UnknownOptionsException("Unhandled options")

    def do(self, text: str) -> str:

        if len(text) == 0:
            return ""

        otext = ""
        i = 0
        while True:
            if i >= len(text):
                break

            if self._jconv.isRegion(text[i]):
                _, ln = self._jconv.convert(text[i:])
                if ln <= 0:  # pragma: no cover
                    otext = otext + text[i]
                    i += 1
                    self._state = False
                elif (i + ln) < len(text):
                    if self._state:
                        otext = otext + text[i : i + ln] + self._separator
                    else:
                        otext = (
                            otext + self._separator + text[i : i + ln] + self._separator
                        )
                        self._state = True
                    i = i + ln
                else:
                    if self._state:
                        otext = otext + text[i : i + ln]
                    else:  # pragma: no cover
                        otext = otext + self._separator + text[i : i + ln]
                    break

            else:
                self._state = False
                otext = otext + text[i]
                i += 1

        return otext


class J2:
    def __init__(self, mode: str = "H", method: str = "Hepburn"):
        self._itaiji = Itaiji()
        self._jconv = JConv()
        if mode == "H":
            self.convert = self.convert_h
        elif mode in ("a", "K"):
            self._hconv = H2(mode, method)
            self.convert = self.convert_nonh
        else:
            self.convert = self.convert_noop

    def isRegion(self, c: str):
        return 0x3400 <= ord(c[0]) < 0xE000 or self._itaiji.haskey(ord(c[0]))

    def convert_h(self, itext) -> Tuple[str, int]:
        return self._jconv.convert(itext)

    def convert_nonh(self, text):
        if not self.isRegion(text[0]):
            return "", 0

        (t, l1) = self.convert_h(text)
        if l1 <= 0:  # pragma: no cover
            return "", 0

        m = 0
        otext = ""

        while True:
            if m >= len(t):
                break
            (s, n) = self._hconv.convert(t[m:])
            if n <= 0:  # pragma: no cover
                m = m + 1
            else:
                m = m + n
                otext = otext + s

        return otext, l1

    def convert_noop(self, text):
        return text[0], 1



import os
import sys

import cpuinfo
import pytest


@pytest.fixture(scope="session", autouse=True)
def dictionary_setup_fixture():
    root_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    if "TOX_ENV_DIR" not in os.environ:
        sys.path.insert(1, os.path.join(root_dir, "src"))
        import kakasidict
        from pykakasi.properties import Configurations

        dpath = os.path.join(root_dir, "build", "lib", "pykakasi", "data")
        print("Generating kanwa dictionary in %s\n" % dpath)
        kanwa = kakasidict.Genkanwadict()
        kanwa.generate_dictionaries(dpath)
        Configurations.data_path = dpath


def pytest_benchmark_update_json(config, benchmarks, output_json):
    """Calculate speed and add as extra_info"""
    for benchmark in output_json["benchmarks"]:
        if "data_size" in benchmark["extra_info"]:
            rate = (
                benchmark["extra_info"].get("data_size", 0.0)
                / benchmark["stats"]["mean"]
            )
            benchmark["extra_info"]["rate"] = rate


def pytest_benchmark_update_machine_info(config, machine_info):
    cpu_info = cpuinfo.get_cpu_info()
    brand = cpu_info.get("brand_raw", None)
    if brand is None:
        brand = "{} core(s) {} CPU ".format(
            cpu_info.get("count", "unknown"), cpu_info.get("arch", "unknown")
        )
    machine_info["cpu"]["brand"] = brand
    machine_info["cpu"]["hz_actual_friendly"] = cpu_info.get(
        "hz_actual_friendly", "unknown"
    )



# -*- coding: utf-8 -*-
# j2.py
#
# Copyright 2011-2019 Hiroshi Miura <miurahr@linux.com>
import functools
import pickle
import threading
from typing import Tuple

from .properties import Configurations


class JConv:
    def __init__(self):
        self._kanwa = Kanwa()
        self._itaiji = Itaiji()

    def isRegion(self, c: str):
        return 0x3400 <= ord(c[0]) < 0xE000 or self._itaiji.haskey(ord(c[0]))

    @functools.lru_cache(maxsize=512)
    def convert(self, itext: str) -> Tuple[str, int]:
        max_len = 0
        Hstr = ""
        text = self._itaiji.convert(itext)
        num_vs = len(itext) - len(text)
        table = self._kanwa.load(text[0])
        if table is None:
            return "", 0
        for (k, v) in table.items():
            length = len(k)
            if len(text) >= length:
                if text.startswith(k):
                    for yomi in v:
                        if max_len < length:
                            Hstr = yomi
                            max_len = length
        for _ in range(
            num_vs
        ):  # when converting string with kanji wit variation selector, calculate max_len again
            if max_len > len(itext):
                break
            elif text[max_len - 1] != itext[max_len - 1]:
                max_len += 1
            elif (
                max_len < num_vs + len(text)
                and max_len <= len(itext)
                and self._is_vschr(itext[max_len])
            ):
                max_len += 1
            else:
                pass
        return (Hstr, max_len)

    def _isCletter(self, literal: str, c: str) -> bool:
        if (0x3041 <= ord(c) <= 0x309F) and (
            literal in self._cl_table[ord(c) - 0x3040]
        ):  # ぁ:= u\3041
            return True
        return False

    def _is_vschr(self, ch):
        return 0x0E0100 <= ord(ch) <= 0x0E1EF or 0xFE00 <= ord(ch) <= 0xFE02


class Itaiji:

    # this class is Borg/Singleton
    _shared_state = {"_itaijidict": None, "_lock": threading.Lock()}

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._itaijidict is None:
            with self._lock:
                if self._itaijidict is None:
                    itaijipath = Configurations.dictpath(Configurations.jisyo_itaiji)
                    with open(itaijipath, "rb") as d:
                        self._itaijidict = pickle.load(d)

    def haskey(self, c):
        return c in self._itaijidict

    def convert(self, text: str) -> str:
        return text.translate(self._itaijidict)


# This class is Borg/Singleton
# It provides same results becase lookup from a static dictionary.
# There is no state rather dictionary dbm.
class Kanwa:
    _shared_state = {"_lock": threading.Lock(), "_jisyo_table": None}

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._jisyo_table is None:
            with self._lock:
                if self._jisyo_table is None:
                    dictpath = Configurations.dictpath(Configurations.jisyo_kanwa)
                    with open(dictpath, "rb") as d:
                        self._jisyo_table = pickle.load(d)

    def load(self, char: str):
        key = ord(char[0])
        return self._jisyo_table.get(key, None)



from .properties import (
    Ch as Ch,
    Configurations as Configurations,
    Convert_Tables as Convert_Tables,
)
from typing import Any, Dict

class IConv:
    def __init__(self) -> None: ...
    def convert(self, otext: str, hira: str) -> Dict[str, str]: ...

class H2:
    convert: Any = ...
    def __init__(self, mode: Any, method: str = ...) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_a(self, text: Any): ...
    def convert_K(self, text: Any): ...
    def convert_noop(self, text: Any): ...

class K2:
    convert: Any = ...
    def __init__(self, mode: Any, method: str = ...) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_a(self, text: Any): ...
    def convert_h(self, text: Any): ...
    def convert_noop(self, text: Any): ...

class Jisyo:
    def __init__(self, dictname: Any) -> None: ...
    def haskey(self, key: Any): ...
    def lookup(self, key: Any): ...
    def maxkeylen(self): ...

class Sym2:
    convert: Any = ...
    def __init__(self, mode: Any) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_a(self, text: Any): ...
    def convert_noop(self, text: Any): ...

class A2:
    convert: Any = ...
    def __init__(self, mode: Any) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_E(self, text: Any): ...
    def convert_noop(self, text: Any): ...



from .properties import Configurations as Configurations
from typing import Any, Tuple

class JConv:
    def __init__(self) -> None: ...
    def isRegion(self, c: str) -> Any: ...
    def convert(self, itext: str) -> Tuple[str, int]: ...

class Itaiji:
    __dict__: Any = ...
    def __new__(cls, *p: Any, **k: Any): ...
    def __init__(self) -> None: ...
    def haskey(self, c: Any): ...
    def convert(self, text: str) -> str: ...

class Kanwa:
    __dict__: Any = ...
    def __new__(cls, *p: Any, **k: Any): ...
    def __init__(self) -> None: ...
    def load(self, char: str) -> Any: ...
